@@ -194,8 +194,7 @@ struct io_cifs *add_list_cifs(struct io_cifs **clist, char *name)
 		}
 		memset(c->cifs_stats[i], 0, CIFS_ST_SIZE);
 	}
-	strncpy(c->name, name, sizeof(c->name));
-	c->name[sizeof(c->name) - 1] = '\0';
+	snprintf(c->name, sizeof(c->name), "%s", name);
 	c->exist = TRUE;
 	c->next = cs;
 
@@ -216,7 +215,7 @@ struct io_cifs *add_list_cifs(struct io_cifs **clist, char *name)
 int read_cifs_stat(int curr)
 {
 	FILE *fp;
-	char line[256];
+	char line[1024];
 	char aux[32];
 	int start = 0;
 	long long unsigned aux_open;
@@ -229,15 +228,14 @@ int read_cifs_stat(int curr)
 	if ((fp = fopen(CIFSSTATS, "r")) == NULL)
 		return 1;
 
-	sprintf(aux, "%%*d) %%%ds",
-		MAX_NAME_LEN < 200 ? MAX_NAME_LEN - 1 : 200);
+	sprintf(aux, "%%*d) %%%ds", MAX_NAME_LEN - 1);
 
 	memset(&scifs, 0, CIFS_ST_SIZE);
 
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		/* Read CIFS directory name */
-		if (isdigit((unsigned char) line[0]) && sscanf(line, aux , name_tmp) == 1) {
+		if (isdigit((unsigned char) line[0]) && sscanf(line, aux, name_tmp) == 1) {
 			if (start) {
 				scifs.fopens = all_open;
 				ci = add_list_cifs(&cifs_list, cifs_name);
@@ -249,8 +247,7 @@ int read_cifs_stat(int curr)
 			else {
 				start = 1;
 			}
-			strncpy(cifs_name, name_tmp, sizeof(cifs_name));
-			cifs_name[sizeof(cifs_name) - 1] = '\0';
+			snprintf(cifs_name, sizeof(cifs_name), "%s", name_tmp);
 			memset(&scifs, 0, CIFS_ST_SIZE);
 		}
 		else {
@@ -751,9 +748,9 @@ int main(int argc, char **argv)
 
 				case 'V':
 					{
-						char *cifsiostat_env[] = {ENV_COLORS,
-									  ENV_COLORS_SGR,
-									  ENV_TIME_FMT};
+						const char *cifsiostat_env[] = {ENV_COLORS,
+										ENV_COLORS_SGR,
+										ENV_TIME_FMT};
 #define CIFSIOSTAT_ENV_NR	3
 						/* Print environment contents, version number and exit */
 						print_version(cifsiostat_env, CIFSIOSTAT_ENV_NR);
@@ -813,7 +810,7 @@ int main(int argc, char **argv)
 	__uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
 			     header.nodename, header.machine, cpu_nr,
-			     DISPLAY_JSON_OUTPUT(xflags))) {
+			     DISPLAY_JSON_OUTPUT(xflags)) > 0) {
 		xflags |= X_D_ISO;
 	}
 
