@@ -36,7 +36,10 @@
 
 #include "version.h"
 #include "common.h"
+
+#ifndef SOURCE_SADC
 #include "ioconf.h"
+#endif /* SOURCE_SADC */
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -84,7 +87,7 @@ char persistent_name_type[MAX_FILE_LEN];
  * @n		Number of environment variables.
  ***************************************************************************
  */
-void print_version(char *env[], int n)
+void print_version(const char *env[], int n)
 {
 	char *e;
 	int i;
@@ -115,23 +118,27 @@ void print_version(char *env[], int n)
  * @rectime	Current local date and time.
  *
  * RETURNS:
- * Value of time in seconds since the Epoch (always in UTC)
+ * Value of time in seconds since the Epoch (always in UTC) on success,
+ * or (time_t) -1 on error.
  ***************************************************************************
  */
 time_t get_xtime(struct tm *rectime, int d_off, int utc)
 {
 	time_t timer;
 
-	timer = __time(NULL);
+	if ((timer = __time(NULL)) == (time_t) -1)
+		return (time_t) -1;
 	timer -= SEC_PER_DAY * d_off;
 
 	if (utc) {
 		/* Get date and time in UTC */
-		gmtime_r(&timer, rectime);
+		if (gmtime_r(&timer, rectime) == NULL)
+			return (time_t) -1;
 	}
 	else {
 		/* Get date and time in local time */
-		localtime_r(&timer, rectime);
+		if (localtime_r(&timer, rectime) == NULL)
+			return (time_t) -1;
 	}
 
 	return timer;
@@ -148,7 +155,7 @@ time_t get_xtime(struct tm *rectime, int d_off, int utc)
  * @rectime	Current date and time.
  *
  * RETURNS:
- * Value of time in seconds since the Epoch.
+ * Value of time in seconds since the Epoch, or (time_t) -1 on error.
  ***************************************************************************
  */
 time_t get_time(struct tm *rectime, int d_off)
@@ -201,7 +208,7 @@ void init_nls(void)
  * TRUE if @name is a device, and FALSE if it's a partition.
  ***************************************************************************
  */
-int is_device(char *sysdev, char *name, int allow_virtual)
+int is_device(const char *sysdev, char *name, int allow_virtual)
 {
 	char syspath[PATH_MAX];
 	char *slash;
@@ -229,6 +236,7 @@ void get_kb_shift(void)
 	/* One can also use getpagesize() to get the size of a page */
 	if ((size = sysconf(_SC_PAGESIZE)) == -1) {
 		perror("sysconf");
+		exit(2);
 	}
 
 	size >>= 10;	/* Assume that a page has a minimum size of 1 kB */
@@ -252,6 +260,7 @@ void get_HZ(void)
 
 	if ((ticks = sysconf(_SC_CLK_TCK)) == -1) {
 		perror("sysconf");
+		exit(2);
 	}
 
 	hz = (unsigned long) ticks;
@@ -291,53 +300,58 @@ void sysstat_panic(const char *function, int error_code)
  * @name	Filename read from /dev/disk/by-id.
  *
  * OUT:
- * @wwn		WWN identifier (8 or 16 hex characters).
+ * @wwn		WWN identifier (16 or 32 hex characters).
+ *		First part saved in *wwn (ie. wwn[0]) and second part (if
+ *		any) in	*(wwn + 1) (ie. wwn[1]).
  * @part-nr	Partition number if applicable.
  *
  * RETURNS:
  * 0 on success, -1 otherwise.
  ***************************************************************************
 */
-int extract_wwnid(char *name, unsigned long long *wwn, unsigned int *part_nr)
+int extract_wwnid(const char *name, unsigned long long *wwn, unsigned int *part_nr)
 {
-	char id[17];
+	char id[WWN_SHORT_LEN + 1];
 	char *s;
-	int wwnlen;
+	int wwnlen = strlen(name);
+
+	if ((name == NULL) || (wwn == NULL) || (part_nr == NULL))
+		return -1;
 
 	*wwn = *(wwn + 1) = 0ULL;
 	*part_nr = 0;
 
 	/* Check name */
-	if (((wwnlen = strlen(name)) < 22) || (strncmp(name, "wwn-0x", 6)))
+	if ((wwnlen < WWN_PREFIX_LEN + WWN_SHORT_LEN) ||
+	    (strncmp(name, WWN_PREFIX, WWN_PREFIX_LEN) != 0))
 		return -1;
 
 	/* Is there a partition number? */
-	if ((s = strstr(name, "-part")) != NULL) {
+	if ((s = strstr(name, PARTITION_SUFFIX)) != NULL) {
 		/* Yes: Get partition number */
-		if (sscanf(s + 5, "%u", part_nr) == 0)
+		if (sscanf(s + strlen(PARTITION_SUFFIX), "%u", part_nr) != 1)
 			return -1;
-		wwnlen = s - name - 6;
+		wwnlen = s - name - WWN_PREFIX_LEN;
 	}
 	else {
-		wwnlen -= 6;	/* Don't count "wwn-0x" */
+		wwnlen -= WWN_PREFIX_LEN;	/* Don't count "wwn-0x" */
 	}
 
 	/* Check WWN length */
-	if ((wwnlen != 16) && (wwnlen != 32))
+	if ((wwnlen != WWN_SHORT_LEN) && (wwnlen != WWN_LONG_LEN))
 		return -1;
 
 	/* Extract first 16 hex chars of WWN */
-	strncpy(id, name + 6, 16);
-	id[16] = '\0';
-	if (sscanf(id, "%llx", wwn) == 0)
+	snprintf(id, sizeof(id), "%s", name + WWN_PREFIX_LEN);
+	if (sscanf(id, "%llx", wwn) != 1)
 		return -1;
 
-	if (strlen(name) < 38)
+	if (wwnlen == WWN_SHORT_LEN)
 		/* This is a short (16 hex chars) WWN id */
 		return 0;
 
 	/* Extract second part of WWN */
-	if (sscanf(name + 22, "%llx", wwn + 1) == 0)
+	if (sscanf(name + WWN_PREFIX_LEN + WWN_SHORT_LEN, "%llx", wwn + 1) != 1)
 		return -1;
 
 	return 0;
@@ -352,7 +366,7 @@ int extract_wwnid(char *name, unsigned long long *wwn, unsigned int *part_nr)
  * @pretty	Pretty name (e.g. sda, sdb3...).
  *
  * OUT:
- * @wwn		WWN identifier (8 or 16 hex characters).
+ * @wwn		WWN identifier (16 or 32 hex characters).
  * @part-nr	Partition number if applicable.
  *
  * RETURNS:
@@ -375,28 +389,27 @@ int get_wwnid_from_pretty(char *pretty, unsigned long long *wwn, unsigned int *p
 	/* Get current id */
 	while ((drd = readdir(dir)) != NULL) {
 
-		if (strncmp(drd->d_name, "wwn-0x", 6))
+		if (strncmp(drd->d_name, WWN_PREFIX, WWN_PREFIX_LEN) != 0)
 			continue;
 
 		/* Get absolute path for current persistent name */
-		snprintf(link, PATH_MAX, "%s/%s", DEV_DISK_BY_ID, drd->d_name);
+		snprintf(link, sizeof(link), "%s/%s", DEV_DISK_BY_ID, drd->d_name);
 
 		/* Persistent name is usually a symlink: Read it... */
-		r = readlink(link, target, PATH_MAX);
-		if ((r <= 0) || (r >= PATH_MAX))
+		r = readlink(link, target, sizeof(target));
+		if ((r <= 0) || (r >= (ssize_t) sizeof(target)))
 			continue;
 
 		target[r] = '\0';
 
 		/* ... and get device pretty name it points at */
 		name = basename(target);
-		if (!name || (name[0] == '\0'))
+		if (!name || (*name == '\0'))
 			continue;
 
-		if (!strncmp(name, pretty, FILENAME_MAX)) {
+		if (strncmp(name, pretty, FILENAME_MAX) == 0) {
 			/* We have found pretty name for current persistent one */
-			strncpy(wwn_name, drd->d_name, MINIMUM(sizeof(wwn_name), sizeof(drd->d_name)));
-			wwn_name[sizeof(wwn_name) - 1] = '\0';
+			snprintf(wwn_name, sizeof(wwn_name), "%s", drd->d_name);
 
 			/* Try to extract WWN */
 			if (!extract_wwnid(wwn_name, wwn, part_nr)) {
@@ -449,15 +462,16 @@ int check_dir(char *dirname)
  */
 size_t mul_check_overflow3(size_t val1, size_t val2, size_t val3)
 {
-	if ((val1 != 0) && (val2 != 0) && (val3 != 0) &&
-	    ((UINT_MAX < val1) ||
-	     (UINT_MAX / val1 < val2) ||
-	     (UINT_MAX / (val1 * val2) < val3))) {
+	if ((val1 != 0) && (val2 != 0) && (val3 != 0)) {
+		if ((val1 > UINT_MAX) ||
+		    (val2 > UINT_MAX / val1) ||
+		    (val3 > UINT_MAX / (val1 * val2))) {
 #ifdef DEBUG
-		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu). Aborting...\n",
-			__FUNCTION__, val1, val2, val3);
+			fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu). Aborting...\n",
+				__FUNCTION__, val1, val2, val3);
 #endif
-		exit(4);
+			exit(4);
+		}
 	}
 
 	return (val1 * val2 * val3);
@@ -479,16 +493,17 @@ size_t mul_check_overflow3(size_t val1, size_t val2, size_t val3)
  */
 size_t mul_check_overflow4(size_t val1, size_t val2, size_t val3, size_t val4)
 {
-	if ((val1 != 0) && (val2 != 0) && (val3 != 0) && (val4 != 0) &&
-	    ((UINT_MAX < val1) ||
-	     (UINT_MAX / val1 < val2) ||
-	     (UINT_MAX / (val1 * val2) < val3) ||
-	     (UINT_MAX / (val1 * val2 * val3) < val4))) {
+	if ((val1 != 0) && (val2 != 0) && (val3 != 0) && (val4 != 0)) {
+		if ((val1 > UINT_MAX) ||
+		    (val2 > UINT_MAX / val1) ||
+		    (val3 > UINT_MAX / (val1 * val2)) ||
+		    (val4 > UINT_MAX / (val1 * val2 * val3))) {
 #ifdef DEBUG
-		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu,%zu). Aborting...\n",
-			__FUNCTION__, val1, val2, val3, val4);
+			fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu,%zu). Aborting...\n",
+				__FUNCTION__, val1, val2, val3, val4);
 #endif
-		exit(4);
+			exit(4);
+		}
 	}
 
 	return (val1 * val2 * val3 * val4);
@@ -519,9 +534,11 @@ unsigned int get_devmap_major(void)
 
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
-		if (strstr(line, "device-mapper")) {
+		if (strstr(line, DEVICE_MAPPER) != NULL) {
 			/* Read device-mapper major number */
-			sscanf(line, "%u", &dm_major);
+			if (sscanf(line, "%u", &dm_major) == 1)
+				break;
+			dm_major = ~0U;	/* Reset to invalid value if sscanf fails */
 		}
 	}
 
@@ -545,7 +562,7 @@ int is_iso_time_fmt(void)
 	if (is_iso < 0) {
 		char *e;
 
-		is_iso = (((e = __getenv(ENV_TIME_FMT)) != NULL) && !strcmp(e, K_ISO));
+		is_iso = (((e = __getenv(ENV_TIME_FMT)) != NULL) && (strcmp(e, K_ISO) == 0));
 	}
 	return is_iso;
 }
@@ -627,23 +644,23 @@ void xprintf(int nr_tab, const char *fmtf, ...)
  *
  * RETURNS:
  * TRUE if S_TIME_FORMAT is set to ISO, or FALSE otherwise.
+ * It returns -1 on error.
  ***************************************************************************
  */
 int set_report_date(struct tm *tm_time, char cur_date[], int sz)
 {
-	if (tm_time == NULL) {
-		strncpy(cur_date, "?/?/?", sz);
-		cur_date[sz - 1] = '\0';
-	}
-	else if (is_iso_time_fmt()) {
-		strftime(cur_date, sz, "%Y-%m-%d", tm_time);
-		return 1;
-	}
-	else {
-		strftime(cur_date, sz, "%x", tm_time);
+	if (tm_time != NULL) {
+		const char *date_format = is_iso_time_fmt() ?
+					  DATE_FORMAT_ISO :
+					  DATE_FORMAT_LOCAL;
+
+		if (strftime(cur_date, sz, date_format, tm_time) != 0)
+			return is_iso_time_fmt();
 	}
 
-	return 0;
+	snprintf(cur_date, sz, "%s", DEFAULT_ERROR_DATE);
+
+	return -1;
 }
 
 /*
@@ -662,6 +679,7 @@ int set_report_date(struct tm *tm_time, char cur_date[], int sz)
  *
  * RETURNS:
  * TRUE if S_TIME_FORMAT is set to ISO, or FALSE otherwise.
+ * It returns -1 on error.
  ***************************************************************************
  */
 int print_gal_header(struct tm *tm_time, char *sysname, char *release,
@@ -712,7 +730,7 @@ int get_win_height(void)
 	 * This default value will be used whenever STDOUT
 	 * is redirected to a pipe or a file and S_REPEAT_HEADER variable is not set
 	 */
-	int rows = 3600 * 24;
+	int rows = DEFAULT_ROWS;
 
 	/* Get number of lines of current terminal */
 	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &win) != -1) {
@@ -722,14 +740,18 @@ int get_win_height(void)
 	}
 	/* STDOUT is not a terminal. Look for S_REPEAT_HEADER variable's value instead */
 	else if ((e = __getenv(ENV_REPEAT_HEADER)) != NULL) {
+		/* Check if the environment variable contains only digits */
 		if (strspn(e, DIGITS) == strlen(e)) {
-			int v = atol(e);
-			if (v > 0) {
-				rows = v;
+			char *endptr;
+			long v = strtol(e, &endptr, 10);
+			if ((v > 0) && (*endptr == '\0')) {
+				rows = (int) v;
 			}
 		}
 	}
-	return rows;
+
+	/* Ensure rows is at least MIN_ROWS */
+	return (rows < MIN_ROWS) ? MIN_ROWS : rows;
 }
 
 /*
@@ -764,11 +786,10 @@ char *device_name(char *name)
 	fprintf(stderr, "Real pathname: %s (%s)\n", resolved_name, name);
 #endif
 
-	if (!strncmp(resolved_name, "/dev/", 5)) {
-		i = 5;
+	if (strncmp(resolved_name, DEV_PREFIX, DEV_PREFIX_LEN) == 0) {
+		i = DEV_PREFIX_LEN;
 	}
-	strncpy(out, resolved_name + i, sizeof(out));
-	out[sizeof(out) - 1] = '\0';
+	snprintf(out, sizeof(out), "%s", resolved_name + i);
 
 	/* Some devices may have a slash in their name (eg. cciss/c0d0...) */
 	while ((slash = strchr(out, '/'))) {
@@ -776,7 +797,6 @@ char *device_name(char *name)
 	}
 
 	free(resolved_name);
-
 	return out;
 }
 
@@ -791,27 +811,28 @@ char *device_name(char *name)
  * String where '\' characters have been escaped.
  ***************************************************************************
  */
-char *escape_bs_char(char *str)
+char *escape_bs_char(const char str[])
 {
 	static char buffer[MAX_NAME_LEN];
 	int i = 0, j = 0;
 
-	while (str[i] != '\0' && j < MAX_NAME_LEN - 1) {
-		if (str[i] == '\\') {
-			if (j < MAX_NAME_LEN - 2) {
-				buffer[j++] = '\\';
-				buffer[j++] = '\\';
+	if (str != NULL) {
+		while (str[i] != '\0' && j < MAX_NAME_LEN - 1) {
+			if (str[i] == '\\') {
+				if (j < MAX_NAME_LEN - 2) {
+					buffer[j++] = '\\';
+					buffer[j++] = '\\';
+				} else {
+					break;
+				}
 			} else {
-				break;
+				buffer[j++] = str[i];
 			}
-		} else {
-			buffer[j++] = str[i];
+			i++;
 		}
-		i++;
 	}
 
 	buffer[j] = '\0';
-
 	return buffer;
 }
 
@@ -932,7 +953,7 @@ char *get_persistent_type_dir(char *type)
 
 	n = snprintf(dir, sizeof(dir), "%s-%s", DEV_DISK_BY, type);
 
-	if ((n >= sizeof(dir)) || access(dir, R_OK)) {
+	if ((n >= sizeof(dir)) || (n < 0) || (access(dir, R_OK) != 0)) {
 		return (NULL);
 	}
 
@@ -959,7 +980,7 @@ char *get_persistent_name_path(char *name)
 	n = snprintf(path, sizeof(path), "%s/%s",
 		     get_persistent_type_dir(persistent_name_type), name);
 
-	if ((n >= sizeof(path)) || access(path, F_OK)) {
+	if ((n >= sizeof(path)) || (n < 0) || (access(path, F_OK) != 0)) {
 		return (NULL);
 	}
 
@@ -990,12 +1011,12 @@ char **get_persistent_names(void)
 	if (n < 0)
 		return (NULL);
 
-	/* If directory is empty, it contains 2 entries: "." and ".." */
+	/* Empty directories contain only "." and ".." */
 	if (n <= 2)
 		/* Free list and return NULL */
 		goto free_list;
 
-	/* Ignore the "." and "..", but keep place for one last NULL. */
+	/* Ignore "." and ".." but keep place for one last NULL (n-2+1 = n-1) */
 	files = (char **) calloc(n - 1, sizeof(char *));
 	if (!files)
 		goto free_list;
@@ -1006,15 +1027,14 @@ char **get_persistent_names(void)
 	 */
 	for (i = 0; i < n; i++) {
 		/* Ignore "." and ".." */
-		if (!strcmp(".", namelist[i]->d_name) ||
-		    !strcmp("..", namelist[i]->d_name))
+		if ((strcmp(namelist[i]->d_name, ".") == 0) ||
+		    (strcmp(namelist[i]->d_name, "..") == 0))
 			continue;
 
-		files[k] = (char *) calloc(strlen(namelist[i]->d_name) + 1, sizeof(char));
+		files[k] = strdup(namelist[i]->d_name);
 		if (!files[k])
 			continue;
-
-		strcpy(files[k++], namelist[i]->d_name);
+		k++;
 	}
 	files[k] = NULL;
 
@@ -1041,7 +1061,7 @@ free_list:
 */
 char *get_persistent_name_from_pretty(char *pretty)
 {
-	int i = -1;
+	int i;
 	ssize_t r;
 	char *link, *name;
 	char **persist_names;
@@ -1055,7 +1075,7 @@ char *get_persistent_name_from_pretty(char *pretty)
 	if (!persist_names)
 		return (NULL);
 
-	while (persist_names[++i]) {
+	for (i = 0; persist_names[i]; i++) {
 
 		/* Get absolute path for current persistent name */
 		link = get_persistent_name_path(persist_names[i]);
@@ -1063,8 +1083,8 @@ char *get_persistent_name_from_pretty(char *pretty)
 			continue;
 
 		/* Persistent name is usually a symlink: Read it... */
-		r = readlink(link, target, PATH_MAX);
-		if ((r <= 0) || (r >= PATH_MAX))
+		r = readlink(link, target, sizeof(target));
+		if ((r <= 0) || (r >= sizeof(target)))
 			continue;
 
 		target[r] = '\0';
@@ -1074,24 +1094,19 @@ char *get_persistent_name_from_pretty(char *pretty)
 		if (!name || (name[0] == '\0'))
 			continue;
 
-		if (!strncmp(name, pretty, FILENAME_MAX)) {
+		if (strncmp(name, pretty, FILENAME_MAX) == 0) {
 			/* We have found pretty name for current persistent one */
-			strncpy(persist_name, persist_names[i], sizeof(persist_name));
-			persist_name[sizeof(persist_name) - 1] = '\0';
+			snprintf(persist_name, sizeof(persist_name), "%s", persist_names[i]);
 			break;
 		}
 	}
 
-	i = -1;
-	while (persist_names[++i]) {
-		free (persist_names[i]);
+	for (i = 0; persist_names[i]; i++) {
+		free(persist_names[i]);
 	}
-	free (persist_names);
+	free(persist_names);
 
-	if (!strlen(persist_name))
-		return (NULL);
-
-	return persist_name;
+	return persist_name[0] ? persist_name : NULL;
 }
 
 /*
@@ -1116,8 +1131,8 @@ char *get_pretty_name_from_persistent(char *persistent)
 		return (NULL);
 
 	/* Persistent name is usually a symlink: Read it... */
-	r = readlink(link, target, PATH_MAX);
-	if ((r <= 0) || (r >= PATH_MAX))
+	r = readlink(link, target, sizeof(target));
+	if ((r <= 0) || (r >= sizeof(target)))
 		return (NULL);
 
 	target[r] = '\0';
@@ -1152,8 +1167,8 @@ char *get_devname_from_sysfs(unsigned int major, unsigned int minor)
 	snprintf(link, sizeof(link), "%s/%u:%u", SYSFS_DEV_BLOCK, major, minor);
 
 	/* Get full path to device knowing its major and minor numbers */
-	r = readlink(link, target, PATH_MAX);
-	if (r <= 0 || r >= PATH_MAX)
+	r = readlink(link, target, sizeof(target));
+	if (r <= 0 || r >= sizeof(target))
 		return (NULL);
 
 	target[r] = '\0';
@@ -1190,10 +1205,10 @@ char *get_devname(unsigned int major, unsigned int minor)
 		return (name);
 
 	name = ioc_name(major, minor);
-	if ((name != NULL) && strcmp(name, K_NODEV))
+	if ((name != NULL) && (strcmp(name, K_NODEV) != 0))
 		return (name);
 
-	snprintf(buf, sizeof(buf), "dev%u-%u", major, minor);
+	snprintf(buf, sizeof(buf), DEF_DEVICE_NAME, major, minor);
 	return (buf);
 }
 
@@ -1263,8 +1278,7 @@ char *get_device_name(unsigned int major, unsigned int minor, unsigned long long
 		}
 	}
 
-	strncpy(dname, dev_name, sizeof(dname));
-	dname[sizeof(dname) - 1] = '\0';
+	snprintf(dname, sizeof(dname), "%s", dev_name);
 
 	while ((bang = strchr(dname, '!'))) {
 		/*
@@ -1321,40 +1335,50 @@ void init_colors(void)
 	for (p = strtok(e, ":"); p; p =strtok(NULL, ":")) {
 
 		len = strlen(p);
-		if ((len > 7) || (len < 3) || (*(p + 1) != '=') ||
-		    (strspn(p + 2, ";0123456789") != (len - 2)))
+		if ((len > MAX_SGR_SEQ_LEN) ||
+		    (len < MIN_SGR_SEQ_LEN) ||
+		    (*(p + 1) != '=') || (strspn(p + 2, SGR_DIGITS) != (len - 2)))
 			/* Ignore malformed codes */
 			continue;
 
 		switch (*p) {
 			case 'M':
 			case 'W':
-				snprintf(sc_percent_warn, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_percent_warn, sizeof(sc_percent_warn),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'X':
 			case 'H':
-				snprintf(sc_percent_xtreme, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_percent_xtreme, sizeof(sc_percent_xtreme),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'Z':
-				snprintf(sc_zero_int_stat, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_zero_int_stat, sizeof(sc_zero_int_stat),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'N':
-				snprintf(sc_int_stat, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_int_stat, sizeof(sc_int_stat),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'I':
-				snprintf(sc_item_name, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_item_name, sizeof(sc_item_name),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'C':
-				snprintf(sc_sa_comment, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_sa_comment, sizeof(sc_sa_comment),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case 'R':
-				snprintf(sc_sa_restart, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_sa_restart, sizeof(sc_sa_restart),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case '+':
-				snprintf(sc_trend_pos, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_trend_pos, sizeof(sc_trend_pos),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 			case '-':
-				snprintf(sc_trend_neg, MAX_SGR_LEN, "\e[%sm", p + 2);
+				snprintf(sc_trend_neg, sizeof(sc_trend_neg),
+					 SGR_ESC_SEQ, p + 2);
 				break;
 		}
 	}
@@ -1709,12 +1733,13 @@ void cprintf_tr(int trend, char *format, char *tstring)
  * 0 if the value has been properly read, 1 otherwise.
  ***************************************************************************
  */
-int parse_valstr(char *s, int max_val, int *val)
+int parse_valstr(const char *s, int max_val, int *val)
 {
-	if (!strlen(s)) {
+	if ((s == NULL) || (*s == '\0')) {
 		*val = -1;
 		return 0;
 	}
+	/* Check that string contains only digits */
 	if (strspn(s, DIGITS) != strlen(s))
 		return 1;
 
@@ -1747,9 +1772,11 @@ int parse_range_values(char *t, int max_val, int *val_low, int *val)
 {
 	char *s, *valstr, range[16];
 
+	if ((t == NULL) || (*t == '\0'))
+		return 1;
+
 	/* Parse value or range of values */
-	strncpy(range, t, 16);
-	range[15] = '\0';
+	snprintf(range, sizeof(range), "%s", t);
 	valstr = t;
 
 	if ((s = strchr(range, '-')) != NULL) {
@@ -1775,6 +1802,7 @@ int parse_range_values(char *t, int max_val, int *val_low, int *val)
 	if (!s) {
 		*val_low = *val;
 	}
+
 	return 0;
 }
 
@@ -1804,14 +1832,14 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 	int i, val_low, val;
 	char *t;
 
-	if (!strcmp(strargv, K_ALL)) {
+	if (strcmp(strargv, K_ALL) == 0) {
 		/* Set bit for every possible values (CPU, IRQ, etc.) */
 		memset(bitmap, ~0, BITMAP_SIZE(max_val));
 		return 0;
 	}
 
 	for (t = strtok(strargv, ","); t; t = strtok(NULL, ",")) {
-		if (!strcmp(t, __K_VALUE0)) {
+		if (strcmp(t, __K_VALUE0) == 0) {
 			/*
 			 * Set bit 0 in bitmap. This may correspond
 			 * to CPU "all" or IRQ "SUM" for example.
@@ -1846,16 +1874,22 @@ void write_sample_timestamp(int tab, struct tm *rectime, uint64_t xflags)
 {
 	char timestamp[TIMESTAMP_LEN];
 
+	if (rectime == NULL)
+		return;
+
 	if (DISPLAY_SEC_EPOCH(xflags)) {
-		snprintf(timestamp, sizeof(timestamp), "%ld", mktime(rectime));
-		timestamp[sizeof(timestamp) - 1] = '\0';
+		snprintf(timestamp, sizeof(timestamp), "%ld", (long) mktime(rectime));
 	}
 	else if (DISPLAY_ISO(xflags)) {
-		strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
+		strftime(timestamp, sizeof(timestamp), DATE_TIME_FORMAT_ISO, rectime);
 	}
 	else {
-		strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
+		strftime(timestamp, sizeof(timestamp), DATE_TIME_FORMAT_LOCAL, rectime);
 	}
+
+	/* Make sure timestamp is null-terminated */
+	timestamp[sizeof(timestamp) - 1] = '\0';
+
 	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		xprintf(tab, "\"timestamp\": \"%s\",", timestamp);
 	}
