@@ -61,9 +61,7 @@ int get_sys_cpu_nr(int highest)
 {
 	DIR *dir;
 	struct dirent *drd;
-	struct stat buf;
-	char line[MAX_PF_NAME];
-	int num_proc, proc_nr = -1;
+	int cpu_id, count = 0;
 
 	/* Open relevant /sys directory */
 	if ((dir = opendir(SYSFS_DEVCPU)) == NULL)
@@ -72,29 +70,24 @@ int get_sys_cpu_nr(int highest)
 	/* Get current file entry */
 	while ((drd = readdir(dir)) != NULL) {
 
-		if (!strncmp(drd->d_name, "cpu", 3) && isdigit(drd->d_name[3])) {
-			snprintf(line, sizeof(line), "%s/%s", SYSFS_DEVCPU, drd->d_name);
-			line[sizeof(line) - 1] = '\0';
-			if (stat(line, &buf) < 0)
+		if ((strncmp(drd->d_name, "cpu", 3) != 0) || !isdigit(drd->d_name[3]))
+			continue;
+		if (highest) {
+			if (sscanf(drd->d_name + 3, "%d", &cpu_id) != 1)
 				continue;
-			if (S_ISDIR(buf.st_mode)) {
-				if (highest) {
-					sscanf(drd->d_name + 3, "%d", &num_proc);
-					if (num_proc > proc_nr) {
-						proc_nr = num_proc;
-					}
-				}
-				else {
-					proc_nr++;
-				}
+			if (cpu_id > count) {
+				count = cpu_id;
 			}
 		}
+		else {
+			count++;
+		}
 	}
 
 	/* Close directory */
 	closedir(dir);
 
-	return (proc_nr + 1);
+	return highest ? count + 1 : count;
 }
 
 /*
@@ -111,8 +104,8 @@ int get_sys_cpu_nr(int highest)
 int get_proc_cpu_nr(void)
 {
 	FILE *fp;
-	char line[16];
-	int num_proc, proc_nr = -1;
+	char line[256];
+	int cpu_id, max_cpu = -1;
 
 	if ((fp = fopen(STAT, "r")) == NULL) {
 		fprintf(stderr, _("Cannot open %s: %s\n"), STAT, strerror(errno));
@@ -121,18 +114,18 @@ int get_proc_cpu_nr(void)
 
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
-		if (strncmp(line, "cpu ", 4) && !strncmp(line, "cpu", 3)) {
-			sscanf(line + 3, "%d", &num_proc);
-			if (num_proc > proc_nr) {
-				proc_nr = num_proc;
-			}
+		if ((strncmp(line, "cpu", 3) != 0) || !isdigit(line[3]))
+			continue;
+		if (sscanf(line + 3, "%d", &cpu_id) != 1)
+			continue;
+		if (cpu_id > max_cpu) {
+			max_cpu = cpu_id;
 		}
 	}
 
 	fclose(fp);
 
-	proc_nr++;
-	return proc_nr;
+	return max_cpu + 1;
 }
 
 /*
@@ -242,7 +235,7 @@ __nr_t get_irqcpu_nr(char *file, int max_nr_irqcpu, int cpu_nr)
 __nr_t get_diskstats_dev_nr(int count_part, int only_used_dev)
 {
 	FILE *fp;
-	char line[256];
+	char line[1024], aux[64];
 	char dev_name[MAX_NAME_LEN];
 	__nr_t dev = 0;
 	int i;
@@ -252,6 +245,8 @@ __nr_t get_diskstats_dev_nr(int count_part, int only_used_dev)
 		/* File non-existent */
 		return 0;
 
+	sprintf(aux, "%%*d %%*d %%%ds %%lu %%*u %%*u %%*u %%lu", MAX_NAME_LEN - 1);
+
 	/*
 	 * Counting devices and partitions is simply a matter of counting
 	 * the number of lines...
@@ -259,8 +254,7 @@ __nr_t get_diskstats_dev_nr(int count_part, int only_used_dev)
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		if (!count_part) {
-			i = sscanf(line, "%*d %*d %127s %lu %*u %*u %*u %lu",
-				   dev_name, &rd_ios, &wr_ios);
+			i = sscanf(line, aux, dev_name, &rd_ios, &wr_ios);
 			if ((i == 2) || !is_device(SLASH_SYS, dev_name, ACCEPT_VIRTUAL_DEVICES))
 				/* It was a partition and not a device */
 				continue;
@@ -437,7 +431,8 @@ __nr_t get_usb_nr(void)
 __nr_t get_filesystem_nr(void)
 {
 	FILE *fp;
-	char line[512], fs_name[MAX_FS_LEN], mountp[256], type[128];
+	char line[1024], aux[16], aux2[16];
+	char fs_name[MAX_FS_LEN], mountp[MAX_FS_LEN * 2], type[MAX_FS_LEN];
 	char *pos = 0, *pos2 = 0;
 	__nr_t fs = 0;
 	int skip, skip_next = 0;
@@ -447,6 +442,9 @@ __nr_t get_filesystem_nr(void)
 		/* File non-existent */
 		return 0;
 
+	sprintf(aux, "%%%ds", MAX_FS_LEN - 1);
+	sprintf(aux2, "%%%ds", (MAX_FS_LEN * 2) - 1);
+
 	/* Get current filesystem */
 	while (fgets(line, sizeof(line), fp) != NULL) {
 		/*
@@ -474,13 +472,13 @@ __nr_t get_filesystem_nr(void)
 			if (pos2 == NULL)
 				continue;
 
-			sscanf(pos2 + 1, "%127s", type);
+			sscanf(pos2 + 1, aux, type);
 			if (strcmp(type, "autofs") == 0)
 				continue;
 
 			/* Read filesystem name and mount point */
-			sscanf(line, "%127s", fs_name);
-			sscanf(pos + 1, "%255s", mountp);
+			sscanf(line, aux, fs_name);
+			sscanf(pos + 1, aux2, mountp);
 
 			/* Replace octal codes */
 			oct2chr(mountp);
