@@ -245,7 +245,6 @@ int get_major_minor_nr(char filename[], int *major, int *minor)
 	char dfile[MAX_PF_NAME];
 
 	snprintf(dfile, sizeof(dfile), "%s%s", filename[0] == '/' ? "" : SLASH_DEV, filename);
-	dfile[sizeof(dfile) - 1] = '\0';
 
 	while ((bang = strchr(dfile, '!'))) {
 		/*
@@ -352,12 +351,11 @@ struct io_device *add_list_device(struct io_device **dlist, char *name, int dtyp
 		if (!dm_name) {
 			dm_name = name;
 		}
-		strncpy(d->name, dm_name, sizeof(d->name) - 1);
+		snprintf(d->name, sizeof(d->name), "%s", dm_name);
 	}
 	else {
-		strncpy(d->name, name, sizeof(d->name));
+		snprintf(d->name, sizeof(d->name), "%s", name);
 	}
-	d->name[MAX_NAME_LEN - 1] = '\0';
 	d->exist = TRUE;
 	d->next = ds;
 
@@ -505,7 +503,6 @@ int read_sysfs_file_stat(char *devname, struct io_stats *ios)
 		/* Read stats for current whole device using /sys/block/ directory */
 		snprintf(dfile, sizeof(dfile), "%s/%s/%s/%s",
 			 SLASH_SYS, __BLOCK, devname, S_STAT);
-		dfile[sizeof(dfile) - 1] = '\0';
 
 		rc = read_sysfs_file_stat_work(dfile, ios);
 	}
@@ -514,7 +511,6 @@ int read_sysfs_file_stat(char *devname, struct io_stats *ios)
 		/* Read stats for current whole device using an alternate /sys directory */
 		snprintf(dfile, sizeof(dfile), "%s/%s/%s/%s",
 			 alt_dir, __BLOCK, devname, S_STAT);
-		dfile[sizeof(dfile) - 1] = '\0';
 
 		rc = read_sysfs_file_stat_work(dfile, ios);
 	}
@@ -545,7 +541,6 @@ int read_sysfs_device_part_stat_work(int curr, char *dname, char *sysdev)
 	char dfile[MAX_PF_NAME], filename[MAX_PF_NAME + 512];
 
 	snprintf(dfile, sizeof(dfile), "%s/%s/%s", sysdev, __BLOCK, dname);
-	dfile[sizeof(dfile) - 1] = '\0';
 
 	/* Open current device directory in /sys/block */
 	if ((dir = __opendir(dfile)) == NULL)
@@ -557,7 +552,6 @@ int read_sysfs_device_part_stat_work(int curr, char *dname, char *sysdev)
 		if (!strcmp(drd->d_name, ".") || !strcmp(drd->d_name, ".."))
 			continue;
 		snprintf(filename, sizeof(filename), "%s/%s/%s", dfile, drd->d_name, S_STAT);
-		filename[sizeof(filename) - 1] = '\0';
 
 		/* Read current partition stats */
 		if (read_sysfs_file_stat_work(filename, &sdev) < 0)
@@ -636,7 +630,6 @@ int read_sysfs_all_devices_stat_work(int curr, char *sysblock)
 		if (!strcmp(drd->d_name, ".") || !strcmp(drd->d_name, ".."))
 			continue;
 		snprintf(dfile, sizeof(dfile), "%s/%s/%s", sysblock, drd->d_name, S_STAT);
-		dfile[sizeof(dfile) - 1] = '\0';
 
 		/* Read current whole device stats */
 		if (read_sysfs_file_stat_work(dfile, &sdev) < 0)
@@ -679,7 +672,6 @@ int read_sysfs_all_devices_stat(int curr)
 		char sysblock[MAX_PF_NAME];
 
 		snprintf(sysblock, sizeof(sysblock), "%s/%s", alt_dir, __BLOCK);
-		sysblock[sizeof(sysblock) - 1] = '\0';
 		/* Read stats from an alternate sys location */
 		rc = read_sysfs_all_devices_stat_work(curr, sysblock);
 	}
@@ -707,7 +699,6 @@ int read_sysfs_part_stat_work(int curr, struct io_device *d, char *sysdev)
 	/* Read stats for device */
 	snprintf(dfile, sizeof(dfile), "%s/%s/%d:%d/%s",
 		 sysdev, __DEV_BLOCK, d->major, d->minor, S_STAT);
-	dfile[sizeof(dfile) - 1] = '\0';
 
 	return read_sysfs_file_stat_work(dfile, d->dev_stats[curr]);
 }
@@ -802,7 +793,7 @@ void read_sysfs_dlist_stat(int curr)
 void read_diskstats_stat_work(int curr, char *diskstats)
 {
 	FILE *fp;
-	char line[256], dev_name[MAX_NAME_LEN];
+	char line[1024], dev_name[MAX_NAME_LEN], aux[128];
 	struct io_device *d;
 	struct io_stats sdev;
 	int i;
@@ -815,12 +806,16 @@ void read_diskstats_stat_work(int curr, char *diskstats)
 	if ((fp = fopen(diskstats, "r")) == NULL)
 		return;
 
+	sprintf(aux, "%%u %%u %%%ds %%lu %%lu %%lu %%lu %%lu %%lu "
+		     "%%lu %%u %%u %%u %%u %%lu %%lu %%lu %%u %%lu %%u",
+		     MAX_NAME_LEN - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		memset(&sdev, 0, sizeof(struct io_stats));
 
 		/* major minor name rio rmerge rsect ruse wio wmerge wsect wuse running use aveq dcio dcmerge dcsect dcuse flio fltm */
-		i = sscanf(line, "%u %u %s %lu %lu %lu %lu %lu %lu %lu %u %u %u %u %lu %lu %lu %u %lu %u",
+		i = sscanf(line, aux,
 			   &major, &minor, dev_name,
 			   &rd_ios, &rd_merges_or_rd_sec, &rd_sec_or_wr_ios, &rd_ticks_or_wr_sec,
 			   &wr_ios, &wr_merges, &wr_sec, &wr_ticks, &ios_pgr, &tot_ticks, &rq_ticks,
@@ -896,7 +891,6 @@ void read_diskstats_stat(int curr)
 		char diskstats[MAX_PF_NAME];
 
 		snprintf(diskstats, sizeof(diskstats), "%s/%s", alt_dir, __DISKSTATS);
-		diskstats[sizeof(diskstats) - 1] = '\0';
 		/* Read stats from an alternate diskstats file */
 		read_diskstats_stat_work(curr, diskstats);
 	}
@@ -2168,8 +2162,7 @@ int main(int argc, char **argv)
 			if (argv[opt++][0] == '+') {
 				flags |= I_D_ALL_DIR;
 			}
-			strncpy(alt_dir, argv[opt++], sizeof(alt_dir));
-			alt_dir[sizeof(alt_dir) - 1] = '\0';
+			snprintf(alt_dir, sizeof(alt_dir), "%s", argv[opt++]);
 			if (!check_dir(alt_dir)) {
 				usage(argv[0]);
 			}
@@ -2182,8 +2175,8 @@ int main(int argc, char **argv)
 			if (strnlen(argv[opt], sizeof(persistent_name_type)) >= sizeof(persistent_name_type) - 1) {
 				usage(argv[0]);
 			}
-			strncpy(persistent_name_type, argv[opt], sizeof(persistent_name_type) - 1);
-			persistent_name_type[sizeof(persistent_name_type) - 1] = '\0';
+			snprintf(persistent_name_type, sizeof(persistent_name_type), "%s",
+				 argv[opt]);
 			strtolower(persistent_name_type);
 			/* Check that this is a valid type of persistent device name */
 			if (!get_persistent_type_dir(persistent_name_type)) {
@@ -2299,10 +2292,10 @@ int main(int argc, char **argv)
 
 				case 'V':
 					{
-						char *iostat_env[] = {ENV_POSIXLY_CORRECT,
-								      ENV_COLORS,
-								      ENV_COLORS_SGR,
-								      ENV_TIME_FMT};
+						const char *iostat_env[] = {ENV_POSIXLY_CORRECT,
+									    ENV_COLORS,
+									    ENV_COLORS_SGR,
+									    ENV_TIME_FMT};
 #define IOSTAT_ENV_NR	4
 						/* Print environment contents, version number and exit */
 						print_version(iostat_env, IOSTAT_ENV_NR);
@@ -2417,7 +2410,7 @@ int main(int argc, char **argv)
 	__uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
 			     header.nodename, header.machine, cpu_nr,
-			     DISPLAY_JSON_OUTPUT(xflags))) {
+			     DISPLAY_JSON_OUTPUT(xflags)) > 0) {
 		xflags |= X_D_ISO;
 	}
 	if (!DISPLAY_JSON_OUTPUT(xflags) &&
