@@ -577,9 +577,10 @@ unsigned long long get_per_cpu_interval(struct stats_cpu *scc,
  */
 void oct2chr(char *str)
 {
-	int i = 0;
-	int j, len;
+	int len, i = 0;
 
+	if (!str)
+		return;
 	len = strlen(str);
 
 	while (i < len - 3) {
@@ -589,12 +590,10 @@ void oct2chr(char *str)
 		    (str[i + 2] >= '0') && (str[i + 2] <= '7') &&
 		    (str[i + 3] >= '0') && (str[i + 3] <= '7')) {
 			/* Octal code found */
-			str[i] = (str[i + 1] - 48) * 64 +
-			         (str[i + 2] - 48) * 8  +
-			         (str[i + 3] - 48);
-			for (j = i + 4; j <= len; j++) {
-				str[j - 3] = str[j];
-			}
+			str[i] = (str[i + 1] - '0') * 64 +
+			         (str[i + 2] - '0') * 8  +
+			         (str[i + 3] - '0');
+			memmove(&str[i + 1], &str[i + 4], len - (i + 3));
 			len -= 3;
 		}
 		i++;
@@ -761,6 +760,7 @@ __nr_t read_vmstat_paging(struct stats_paging *st_paging)
 	FILE *fp;
 	char line[128];
 	unsigned long pgtmp;
+	short pg_start = FALSE, pg_end = FALSE;
 
 	if ((fp = fopen(VMSTAT, "r")) == NULL)
 		return 0;
@@ -791,7 +791,8 @@ __nr_t read_vmstat_paging(struct stats_paging *st_paging)
 			/* Read number of pages freed by the system */
 			sscanf(line + 7, "%lu", &st_paging->pgfree);
 		}
-		else if (!strncmp(line, "pgsteal_", 8)) {
+		else if (!strncmp(line, "pgsteal_", 8) && !pg_end) {
+			pg_start = TRUE;
 			/* Read number of pages stolen by the system */
 			sscanf(strchr(line, ' '), "%lu", &pgtmp);
 			st_paging->pgsteal += pgtmp;
@@ -814,6 +815,15 @@ __nr_t read_vmstat_paging(struct stats_paging *st_paging)
 			sscanf(strchr(line, ' '), "%lu", &pgtmp);
 			st_paging->pgdemote += pgtmp;
 		}
+
+		/*
+		 * Read only the first pg_steal_* values in /proc/vmstat
+		 * (i.e. the "reclaimer": pgsteal_kswapd, pgsteal_direct, etc.)
+		 * but not the type of memory reclaimed (pgsteal_anon, pgsteal_file).
+		 */
+		if (strncmp(line, "pgsteal_", 8) && pg_start) {
+			pg_end = TRUE;
+		}
 	}
 
 	fclose(fp);
@@ -837,7 +847,7 @@ __nr_t read_vmstat_paging(struct stats_paging *st_paging)
 __nr_t read_diskstats_io(struct stats_io *st_io)
 {
 	FILE *fp;
-	char line[1024];
+	char line[1024], aux[128];
 	char dev_name[MAX_NAME_LEN];
 	unsigned int major, minor;
 	unsigned long rd_ios, wr_ios, dc_ios;
@@ -846,17 +856,16 @@ __nr_t read_diskstats_io(struct stats_io *st_io)
 	if ((fp = fopen(DISKSTATS, "r")) == NULL)
 		return 0;
 
+	sprintf(aux, "%%u %%u %%%ds %%lu %%*u %%lu %%*u %%lu %%*u "
+		     "%%lu %%*u %%*u %%*u %%*u %%lu %%*u %%lu",
+		     MAX_NAME_LEN - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		/* Discard I/O stats may be not available */
 		dc_ios = dc_sec = 0;
 
-		if (sscanf(line,
-			   "%u %u %127s "
-			   "%lu %*u %lu %*u "
-			   "%lu %*u %lu %*u "
-			   "%*u %*u %*u "
-			   "%lu %*u %lu",
+		if (sscanf(line, aux,
 			   &major, &minor, dev_name,
 			   &rd_ios, &rd_sec,
 			   &wr_ios, &wr_sec,
@@ -906,7 +915,7 @@ __nr_t read_diskstats_disk(struct stats_disk *st_disk, __nr_t nr_alloc,
 			   int read_part)
 {
 	FILE *fp;
-	char line[1024];
+	char line[1024], aux[128];
 	char dev_name[MAX_NAME_LEN];
 	struct stats_disk *st_disk_i;
 	unsigned int major, minor, rd_ticks, wr_ticks, dc_ticks, tot_ticks, rq_ticks, part_nr;
@@ -917,17 +926,16 @@ __nr_t read_diskstats_disk(struct stats_disk *st_disk, __nr_t nr_alloc,
 	if ((fp = fopen(DISKSTATS, "r")) == NULL)
 		return 0;
 
+	sprintf(aux, "%%u %%u %%%ds %%lu %%*u %%lu %%u %%lu %%*u %%lu %%u "
+		     "%%*u %%u %%u %%lu %%*u %%lu %%u",
+		     MAX_NAME_LEN - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		/* Discard I/O stats may be not available */
 		dc_ios = dc_sec = dc_ticks = 0;
 
-		if (sscanf(line,
-			   "%u %u %127s "
-			   "%lu %*u %lu %u "
-			   "%lu %*u %lu %u "
-			   "%*u %u %u "
-			   "%lu %*u %lu %u",
+		if (sscanf(line, aux,
 			   &major, &minor, dev_name,
 			   &rd_ios, &rd_sec, &rd_ticks,
 			   &wr_ios, &wr_sec, &wr_ticks,
@@ -1138,7 +1146,7 @@ __nr_t read_net_dev(struct stats_net_dev *st_net_dev, __nr_t nr_alloc)
 {
 	FILE *fp;
 	struct stats_net_dev *st_net_dev_i;
-	char line[256];
+	char line[256], aux[16];
 	char iface[MAX_IFACE_LEN];
 	__nr_t dev_read = 0;
 	int pos;
@@ -1146,6 +1154,8 @@ __nr_t read_net_dev(struct stats_net_dev *st_net_dev, __nr_t nr_alloc)
 	if ((fp = fopen(NET_DEV, "r")) == NULL)
 		return 0;
 
+	sprintf(aux, "%%%ds", MAX_IFACE_LEN - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		pos = strcspn(line, ":");
@@ -1159,7 +1169,7 @@ __nr_t read_net_dev(struct stats_net_dev *st_net_dev, __nr_t nr_alloc)
 			st_net_dev_i = st_net_dev + dev_read++;
 			strncpy(iface, line, MINIMUM(pos, sizeof(iface) - 1));
 			iface[MINIMUM(pos, sizeof(iface) - 1)] = '\0';
-			sscanf(iface, "%15s", st_net_dev_i->interface); /* Skip heading spaces */
+			sscanf(iface, aux, st_net_dev_i->interface); /* Skip heading spaces */
 			sscanf(line + pos + 1, "%llu %llu %*u %*u %*u %*u %llu %llu %llu %llu "
 			       "%*u %*u %*u %*u %*u %llu",
 			       &st_net_dev_i->rx_bytes,
@@ -1192,9 +1202,11 @@ void read_if_info(struct stats_net_dev *st_net_dev, int nbr)
 {
 	FILE *fp;
 	struct stats_net_dev *st_net_dev_i;
-	char filename[128], duplex[32];
+	char filename[128], duplex[32], aux[16];
 	int dev, n;
 
+	sprintf(aux, "%%%ds", (int) sizeof(duplex) - 1);
+
 	for (dev = 0; dev < nbr; dev++) {
 
 		st_net_dev_i = st_net_dev + dev;
@@ -1206,7 +1218,7 @@ void read_if_info(struct stats_net_dev *st_net_dev, int nbr)
 			/* Cannot read NIC duplex */
 			continue;
 
-		n = fscanf(fp, "%31s", duplex);
+		n = fscanf(fp, aux, duplex);
 
 		fclose(fp);
 
@@ -1261,7 +1273,7 @@ __nr_t read_net_edev(struct stats_net_edev *st_net_edev, __nr_t nr_alloc)
 {
 	FILE *fp;
 	struct stats_net_edev *st_net_edev_i;
-	static char line[256];
+	static char line[256], aux[16];
 	char iface[MAX_IFACE_LEN];
 	__nr_t dev_read = 0;
 	int pos;
@@ -1269,6 +1281,8 @@ __nr_t read_net_edev(struct stats_net_edev *st_net_edev, __nr_t nr_alloc)
 	if ((fp = fopen(NET_DEV, "r")) == NULL)
 		return 0;
 
+	sprintf(aux, "%%%ds", MAX_IFACE_LEN - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 
 		pos = strcspn(line, ":");
@@ -1282,7 +1296,7 @@ __nr_t read_net_edev(struct stats_net_edev *st_net_edev, __nr_t nr_alloc)
 			st_net_edev_i = st_net_edev + dev_read++;
 			strncpy(iface, line, MINIMUM(pos, sizeof(iface) - 1));
 			iface[MINIMUM(pos, sizeof(iface) - 1)] = '\0';
-			sscanf(iface, "%15s", st_net_edev_i->interface); /* Skip heading spaces */
+			sscanf(iface, aux, st_net_edev_i->interface); /* Skip heading spaces */
 			sscanf(line + pos + 1, "%*u %*u %llu %llu %llu %llu %*u %*u %*u %*u "
 			       "%llu %llu %llu %llu %llu",
 			       &st_net_edev_i->rx_errors,
@@ -2375,7 +2389,7 @@ int read_time_in_state(struct stats_pwr_wghfreq *st_pwr_wghfreq, int cpu_nr, int
 	unsigned long freq;
 	unsigned long long time_in_state;
 
-	snprintf(filename, MAX_PF_NAME, "%s/cpu%d/%s",
+	snprintf(filename, sizeof(filename), "%s/cpu%d/%s",
 		 SYSFS_DEVCPU, cpu_nr, SYSFS_TIME_IN_STATE);
 	if ((fp = fopen(filename, "r")) == NULL)
 		return 0;
@@ -2480,7 +2494,7 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	sscanf(usb_device, "%u", &st_pwr_usb->bus_nr);
 
 	/* Read USB device vendor ID */
-	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
+	snprintf(filename, sizeof(filename), "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_IDVENDOR);
 	if ((fp = fopen(filename, "r")) != NULL) {
 		rc = fscanf(fp, "%x",
@@ -2492,7 +2506,7 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	}
 
 	/* Read USB device product ID */
-	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
+	snprintf(filename, sizeof(filename), "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_IDPRODUCT);
 	if ((fp = fopen(filename, "r")) != NULL) {
 		rc = fscanf(fp, "%x",
@@ -2504,7 +2518,7 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	}
 
 	/* Read USB device max power consumption */
-	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
+	snprintf(filename, sizeof(filename), "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_BMAXPOWER);
 	if ((fp = fopen(filename, "r")) != NULL) {
 		rc = fscanf(fp, "%u",
@@ -2516,7 +2530,7 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	}
 
 	/* Read USB device manufacturer */
-	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
+	snprintf(filename, sizeof(filename), "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_MANUFACTURER);
 	if ((fp = fopen(filename, "r")) != NULL) {
 		rs = fgets(st_pwr_usb->manufacturer,
@@ -2530,7 +2544,7 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	}
 
 	/* Read USB device product */
-	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
+	snprintf(filename, sizeof(filename), "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_PRODUCT);
 	if ((fp = fopen(filename, "r")) != NULL) {
 		rs = fgets(st_pwr_usb->product,
@@ -2611,7 +2625,8 @@ __nr_t read_bus_usb_dev(struct stats_pwr_usb *st_pwr_usb, __nr_t nr_alloc)
 __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 {
 	FILE *fp;
-	char line[512], fs_name[MAX_FS_LEN], mountp[256], type[128];
+	char line[1024], aux[16], aux2[16];
+	char fs_name[MAX_FS_LEN], mountp[MAX_FS_LEN * 2], type[MAX_FS_LEN];
 	int skip, skip_next = 0, fs;
 	char *pos = 0, *pos2 = 0;
 	__nr_t fs_read = 0;
@@ -2621,6 +2636,9 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 	if ((fp = fopen(MTAB, "r")) == NULL)
 		return 0;
 
+	sprintf(aux, "%%%ds", MAX_FS_LEN - 1);
+	sprintf(aux2, "%%%ds", (MAX_FS_LEN * 2) - 1);
+
 	while (fgets(line, sizeof(line), fp) != NULL) {
 		/*
 		 * Ignore line if the preceding line did not contain '\n'.
@@ -2647,12 +2665,12 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			if (pos2 == NULL)
 				continue;
 
-			sscanf(pos2 + 1, "%127s", type);
+			sscanf(pos2 + 1, aux, type);
 			if (strcmp(type, "autofs") == 0)
 				continue;
 
 			/* Read current filesystem name */
-			sscanf(line, "%127s", fs_name);
+			sscanf(line, aux, fs_name);
 			/*
 			 * And now read the corresponding mount point.
 			 * Read fs name and mount point in two distinct operations,
@@ -2663,7 +2681,7 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			 * from the fs name. This would result in a bogus name
 			 * and following statvfs() function would always fail.
 			 */
-			sscanf(pos + 1, "%255s", mountp);
+			sscanf(pos + 1, aux2, mountp);
 
 			/* Replace octal codes */
 			oct2chr(mountp);
@@ -2698,8 +2716,12 @@ __nr_t read_filesystem(struct stats_filesystem *st_filesystem, __nr_t nr_alloc)
 			st_filesystem_i->f_bavail = (unsigned long long) buf.f_bavail * (unsigned long long) buf.f_frsize;
 			st_filesystem_i->f_files  = (unsigned long long) buf.f_files;
 			st_filesystem_i->f_ffree  = (unsigned long long) buf.f_ffree;
-			strncpy(st_filesystem_i->fs_name, fs_name, sizeof(st_filesystem_i->fs_name));
-			st_filesystem_i->fs_name[sizeof(st_filesystem_i->fs_name) - 1] = '\0';
+			snprintf(st_filesystem_i->fs_name, sizeof(st_filesystem_i->fs_name), "%s",
+				 fs_name);
+			/*
+			 * Use strncpy() insteaf of snprintf() because
+			 * src and dest strings are of different size.
+			 */
 			strncpy(st_filesystem_i->mountp, mountp, sizeof(st_filesystem_i->mountp));
 			st_filesystem_i->mountp[sizeof(st_filesystem_i->mountp) - 1] = '\0';
 		}
@@ -2754,7 +2776,7 @@ __nr_t read_fchost(struct stats_fchost *st_fc, __nr_t nr_alloc)
 				break;
 			}
 
-			snprintf(fcstat_filename, MAX_PF_NAME, FC_RX_FRAMES,
+			snprintf(fcstat_filename, sizeof(fcstat_filename), FC_RX_FRAMES,
 				 SYSFS_FCHOST, drd->d_name);
 			if ((fp = fopen(fcstat_filename, "r"))) {
 				if (fgets(line, sizeof(line), fp)) {
@@ -2763,7 +2785,7 @@ __nr_t read_fchost(struct stats_fchost *st_fc, __nr_t nr_alloc)
 				fclose(fp);
 			}
 
-			snprintf(fcstat_filename, MAX_PF_NAME, FC_TX_FRAMES,
+			snprintf(fcstat_filename, sizeof(fcstat_filename), FC_TX_FRAMES,
 				 SYSFS_FCHOST, drd->d_name);
 			if ((fp = fopen(fcstat_filename, "r"))) {
 				if (fgets(line, sizeof(line), fp)) {
@@ -2772,7 +2794,7 @@ __nr_t read_fchost(struct stats_fchost *st_fc, __nr_t nr_alloc)
 				fclose(fp);
 			}
 
-			snprintf(fcstat_filename, MAX_PF_NAME, FC_RX_WORDS,
+			snprintf(fcstat_filename, sizeof(fcstat_filename), FC_RX_WORDS,
 				 SYSFS_FCHOST, drd->d_name);
 			if ((fp = fopen(fcstat_filename, "r"))) {
 				if (fgets(line, sizeof(line), fp)) {
@@ -2781,7 +2803,7 @@ __nr_t read_fchost(struct stats_fchost *st_fc, __nr_t nr_alloc)
 				fclose(fp);
 			}
 
-			snprintf(fcstat_filename, MAX_PF_NAME, FC_TX_WORDS,
+			snprintf(fcstat_filename, sizeof(fcstat_filename), FC_TX_WORDS,
 				 SYSFS_FCHOST, drd->d_name);
 			if ((fp = fopen(fcstat_filename, "r"))) {
 				if (fgets(line, sizeof(line), fp)) {
@@ -3083,7 +3105,7 @@ __nr_t read_bat(struct stats_pwr_bat *st_bat, __nr_t nr_alloc)
 			sscanf(drd->d_name + 3, "%u", &bat_id);
 
 			/* Read battery capcity */
-			snprintf(bat_filename, MAX_PF_NAME, BAT_CAPACITY,
+			snprintf(bat_filename, sizeof(bat_filename), BAT_CAPACITY,
 				 SYSFS_PWR_SUPPLY, drd->d_name);
 			if ((fp = fopen(bat_filename, "r"))) {
 				if (fgets(line, sizeof(line), fp)) {
@@ -3093,7 +3115,7 @@ __nr_t read_bat(struct stats_pwr_bat *st_bat, __nr_t nr_alloc)
 			}
 
 			/* Read battery status */
-			snprintf(bat_filename, MAX_PF_NAME, BAT_STATUS,
+			snprintf(bat_filename, sizeof(bat_filename), BAT_STATUS,
 				 SYSFS_PWR_SUPPLY, drd->d_name);
 			if ((fp = fopen(bat_filename, "r"))) {
 				fgets(status, sizeof(status), fp);
