@@ -268,13 +268,12 @@ void init_structures(void)
  * @ltemp	Argument value.
  ***************************************************************************
  */
-void salloc(int i, char *ltemp)
+void salloc(int i, const char *ltemp)
 {
-	if ((args[i] = (char *) malloc(strlen(ltemp) + 1)) == NULL) {
-		perror("malloc");
+	if ((args[i] = strdup(ltemp)) == NULL) {
+		perror("strdup");
 		exit(4);
 	}
-	strcpy(args[i], ltemp);
 }
 
 /*
@@ -387,11 +386,9 @@ void write_stats_avg(int curr, int read_from_file, unsigned int act_id)
 	/* Interval value in 1/100th of a second */
 	itv = get_interval(record_hdr[2].uptime_cs, record_hdr[curr].uptime_cs);
 
-	strncpy(timestamp[curr], _("Average:"), sizeof(timestamp[curr]));
-	timestamp[curr][sizeof(timestamp[curr]) - 1] = '\0';
+	snprintf(timestamp[curr], sizeof(timestamp[curr]), "%s", _("Average:"));
 	if (DISPLAY_MINMAX(flags)) {
-		strncpy(timestamp[!curr], _("Summary:"), sizeof(timestamp[!curr]));
-		timestamp[!curr][sizeof(timestamp[!curr]) - 1] = '\0';
+		snprintf(timestamp[!curr], sizeof(timestamp[!curr]), "%s", _("Summary:"));
 	}
 	else {
 		memcpy(timestamp[!curr], timestamp[curr], sizeof(timestamp[!curr]));
@@ -628,9 +625,10 @@ __printf_funct_t print_sar_restart(int *tab, int action, char *cur_date, char *c
 	char restart[64];
 
 	printf("\n%-11s", cur_time);
-	sprintf(restart, "  LINUX RESTART\t(%u CPU)\n",
+	sprintf(restart, "  LINUX RESTART\t(%u CPU)",
 		file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
 	cprintf_s(IS_RESTART, "%s", restart);
+	printf("\n");
 
 }
 
@@ -654,7 +652,8 @@ __print_funct_t print_sar_comment(int *tab, int action, char *cur_date, char *cu
 				  struct record_header *record_hdr)
 {
 	printf("%-11s", cur_time);
-	cprintf_s(IS_COMMENT, "  COM %s\n", comment);
+	cprintf_s(IS_COMMENT, "  COM %s", comment);
+	printf("\n");
 }
 
 /*
@@ -922,7 +921,8 @@ void read_header_data(void)
 			/*
 			 * No data (0 byte) have been sent by sadc.
 			 * This is probably because no activities have been collected
-			 * ("Requested activities not available"). In this case, don't
+			 * ("Requested activities not available") or because sadc
+			 * couldn't have been properly started. In this case, don't
 			 * display an error message: Exit now.
 			 */
 			exit(3);
@@ -1422,11 +1422,11 @@ int main(int argc, char **argv)
 		}
 
 		else if (!strcmp(argv[opt], "-V")) {
-			char *sar_env[] = {ENV_COLORS,
-					   ENV_COLORS_SGR,
-					   ENV_REPEAT_HEADER,
-					   ENV_TIME_DEFTM,
-					   ENV_TIME_FMT};
+			const char *sar_env[] = {ENV_COLORS,
+						 ENV_COLORS_SGR,
+						 ENV_REPEAT_HEADER,
+						 ENV_TIME_DEFTM,
+						 ENV_TIME_FMT};
 #define SAR_ENV_NR	5
 			print_version(sar_env, SAR_ENV_NR);
 		}
@@ -1439,8 +1439,7 @@ int main(int argc, char **argv)
 			/* Save stats to a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
-				strncpy(to_file, argv[opt++], sizeof(to_file));
-				to_file[sizeof(to_file) - 1] = '\0';
+				snprintf(to_file, sizeof(to_file), "%s", argv[opt++]);
 			}
 			else {
 				strcpy(to_file, "-");
@@ -1455,8 +1454,7 @@ int main(int argc, char **argv)
 			/* Read stats from a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
-				strncpy(from_file, argv[opt++], sizeof(from_file));
-				from_file[sizeof(from_file) - 1] = '\0';
+				snprintf(from_file, sizeof(from_file), "%s", argv[opt++]);
 				/* Check if this is an alternate directory for sa files */
 				check_alt_sa_dir(from_file, day_offset, -1);
 			}
@@ -1667,6 +1665,7 @@ int main(int argc, char **argv)
 
 	case -1:
 		perror("fork");
+		CLOSE_ALL(fd);
 		exit(4);
 		break;
 
