@@ -820,9 +820,9 @@ int main(int argc, char **argv)
 
 				case 'V':
 					{
-						char *tapestat_env[] = {ENV_COLORS,
-									ENV_COLORS_SGR,
-									ENV_TIME_FMT};
+						const char *tapestat_env[] = {ENV_COLORS,
+									      ENV_COLORS_SGR,
+									      ENV_TIME_FMT};
 #define TAPESTAT_ENV_NR	3
 						/* Print environment contents, version number and exit */
 						print_version(tapestat_env, TAPESTAT_ENV_NR);
@@ -881,7 +881,7 @@ int main(int argc, char **argv)
 	__uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
 			     header.nodename, header.machine, cpu_nr,
-			     DISPLAY_JSON_OUTPUT(xflags))) {
+			     DISPLAY_JSON_OUTPUT(xflags)) > 0) {
 		xflags |= X_D_ISO;
 	}
 	if (!DISPLAY_JSON_OUTPUT(xflags) &&
