@@ -1,6 +1,6 @@
 /*
  * xml_stats.c: Functions used by sadf to display statistics in XML.
- * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -25,7 +25,6 @@
 
 #include "sa.h"
 #include "ioconf.h"
-#include "xml_stats.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -159,8 +158,8 @@ __print_funct_t xml_print_cpu_stats(struct activity *a, int curr, int tab,
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* Don't display CPU */
 			continue;
 
@@ -354,7 +353,7 @@ __print_funct_t xml_print_irq_stats(struct activity *a, int curr, int tab,
 										  + i * a->msize);
 
 			/* Should current CPU (including CPU "all") be displayed? */
-			if (masked_cpu_bitmap[c >> 3] & (1 << (c & 0x07)))
+			if (IS_CPU_SET(masked_cpu_bitmap, c))
 				/* No */
 				continue;
 
@@ -430,7 +429,8 @@ __print_funct_t xml_print_paging_stats(struct activity *a, int curr, int tab,
 		"pgscank=\"%.2f\" "
 		"pgscand=\"%.2f\" "
 		"pgsteal=\"%.2f\" "
-		"vmeff-percent=\"%.2f\"/>",
+		"pgprom=\"%.2f\" "
+		"pgdem=\"%.2f\"/>",
 		S_VALUE(spp->pgpgin,        spc->pgpgin,        itv),
 		S_VALUE(spp->pgpgout,       spc->pgpgout,       itv),
 		S_VALUE(spp->pgfault,       spc->pgfault,       itv),
@@ -439,11 +439,8 @@ __print_funct_t xml_print_paging_stats(struct activity *a, int curr, int tab,
 		S_VALUE(spp->pgscan_kswapd, spc->pgscan_kswapd, itv),
 		S_VALUE(spp->pgscan_direct, spc->pgscan_direct, itv),
 		S_VALUE(spp->pgsteal,       spc->pgsteal,       itv),
-		(spc->pgscan_kswapd + spc->pgscan_direct -
-		 spp->pgscan_kswapd - spp->pgscan_direct) ?
-		SP_VALUE(spp->pgsteal, spc->pgsteal,
-			 spc->pgscan_kswapd + spc->pgscan_direct -
-			 spp->pgscan_kswapd - spp->pgscan_direct) : 0.0);
+		S_VALUE(spp->pgpromote,     spc->pgpromote,     itv),
+		S_VALUE(spp->pgdemote,      spc->pgdemote,      itv));
 }
 
 /*
@@ -497,6 +494,85 @@ __print_funct_t xml_print_io_stats(struct activity *a, int curr, int tab,
 	xprintf(--tab, "</io>");
 }
 
+/*
+ * **************************************************************************
+ * Display RAM memory utilization in XML.
+ *
+ * IN:
+ * @smc		Structure with statistics.
+ * @dispall	TRUE if all memory fields should be displayed.
+ * @tab		Indentation in XML output.
+ ***************************************************************************
+ */
+void xml_print_ram_memory_stats(struct stats_memory *smc, int dispall, int *tab)
+{
+	xprintf(++(*tab), "<memfree>%llu</memfree>", smc->frmkb);
+
+	xprintf(*tab, "<avail>%llu</avail>", smc->availablekb);
+
+	xprintf(*tab, "<memused>%llu</memused>", smc->tlmkb - smc->availablekb);
+
+	xprintf(*tab, "<memused-percent>%.2f</memused-percent>",
+		smc->tlmkb ? SP_VALUE(smc->availablekb, smc->tlmkb, smc->tlmkb)
+			   : 0.0);
+
+	xprintf(*tab, "<buffers>%llu</buffers>", smc->bufkb);
+
+	xprintf(*tab, "<cached>%llu</cached>", smc->camkb);
+
+	xprintf(*tab, "<commit>%llu</commit>", smc->comkb);
+
+	xprintf(*tab, "<commit-percent>%.2f</commit-percent>",
+		(smc->tlmkb + smc->tlskb) ? SP_VALUE(0, smc->comkb, smc->tlmkb + smc->tlskb)
+					  : 0.0);
+
+	xprintf(*tab, "<active>%llu</active>", smc->activekb);
+
+	xprintf(*tab, "<inactive>%llu</inactive>", smc->inactkb);
+
+	xprintf(*tab, "<dirty>%llu</dirty>", smc->dirtykb);
+
+	xprintf((*tab)--, "<shared>%llu</shared>", smc->shmemkb);
+
+	if (dispall) {
+		xprintf(++(*tab), "<anonpg>%llu</anonpg>", smc->anonpgkb);
+
+		xprintf(*tab, "<slab>%llu</slab>", smc->slabkb);
+
+		xprintf(*tab, "<kstack>%llu</kstack>", smc->kstackkb);
+
+		xprintf(*tab, "<pgtbl>%llu</pgtbl>", smc->pgtblkb);
+
+		xprintf((*tab)--, "<vmused>%llu</vmused>", smc->vmusedkb);
+	}
+}
+
+/*
+ * **************************************************************************
+ * Display swap memory utilization in XML.
+ *
+ * IN:
+ * @smc		Structure with statistics.
+ * @tab		Indentation in XML output.
+ ***************************************************************************
+ */
+void xml_print_swap_memory_stats(struct stats_memory *smc, int *tab)
+{
+	xprintf(++(*tab), "<swpfree>%llu</swpfree>", smc->frskb);
+
+	xprintf(*tab, "<swpused>%llu</swpused>", smc->tlskb - smc->frskb);
+
+	xprintf(*tab, "<swpused-percent>%.2f</swpused-percent>",
+		smc->tlskb ? SP_VALUE(smc->frskb, smc->tlskb, smc->tlskb)
+			   : 0.0);
+
+	xprintf(*tab, "<swpcad>%llu</swpcad>", smc->caskb);
+
+	xprintf((*tab)--, "<swpcad-percent>%.2f</swpcad-percent>",
+		(smc->tlskb - smc->frskb) ? SP_VALUE(0, smc->caskb, smc->tlskb - smc->frskb)
+					  : 0.0);
+}
+
 /*
  ***************************************************************************
  * Display memory statistics in XML.
@@ -513,92 +589,15 @@ __print_funct_t xml_print_memory_stats(struct activity *a, int curr, int tab,
 {
 	struct stats_memory
 		*smc = (struct stats_memory *) a->buf[curr];
-	unsigned long long nousedmem;
 
 	xprintf(tab, "<memory unit=\"kB\">");
 
 	if (DISPLAY_MEMORY(a->opt_flags)) {
-
-		nousedmem = smc->frmkb + smc->bufkb + smc->camkb + smc->slabkb;
-		if (nousedmem > smc->tlmkb) {
-			nousedmem = smc->tlmkb;
-		}
-
-		xprintf(++tab, "<memfree>%llu</memfree>",
-			smc->frmkb);
-
-		xprintf(tab, "<avail>%llu</avail>",
-			smc->availablekb);
-
-		xprintf(tab, "<memused>%llu</memused>",
-			smc->tlmkb - nousedmem);
-
-		xprintf(tab, "<memused-percent>%.2f</memused-percent>",
-			smc->tlmkb ?
-			SP_VALUE(nousedmem, smc->tlmkb, smc->tlmkb) :
-			0.0);
-
-		xprintf(tab, "<buffers>%llu</buffers>",
-			smc->bufkb);
-
-		xprintf(tab, "<cached>%llu</cached>",
-			smc->camkb);
-
-		xprintf(tab, "<commit>%llu</commit>",
-			smc->comkb);
-
-		xprintf(tab, "<commit-percent>%.2f</commit-percent>",
-			(smc->tlmkb + smc->tlskb) ?
-			SP_VALUE(0, smc->comkb, smc->tlmkb + smc->tlskb) :
-			0.0);
-
-		xprintf(tab, "<active>%llu</active>",
-			smc->activekb);
-
-		xprintf(tab, "<inactive>%llu</inactive>",
-			smc->inactkb);
-
-		xprintf(tab--, "<dirty>%llu</dirty>",
-			smc->dirtykb);
-
-		if (DISPLAY_MEM_ALL(a->opt_flags)) {
-			xprintf(++tab, "<anonpg>%llu</anonpg>",
-				smc->anonpgkb);
-
-			xprintf(tab, "<slab>%llu</slab>",
-				smc->slabkb);
-
-			xprintf(tab, "<kstack>%llu</kstack>",
-				smc->kstackkb);
-
-			xprintf(tab, "<pgtbl>%llu</pgtbl>",
-				smc->pgtblkb);
-
-			xprintf(tab--, "<vmused>%llu</vmused>",
-				smc->vmusedkb);
-		}
+		xml_print_ram_memory_stats(smc, DISPLAY_MEM_ALL(a->opt_flags), &tab);
 	}
 
 	if (DISPLAY_SWAP(a->opt_flags)) {
-
-		xprintf(++tab, "<swpfree>%llu</swpfree>",
-			smc->frskb);
-
-		xprintf(tab, "<swpused>%llu</swpused>",
-			smc->tlskb - smc->frskb);
-
-		xprintf(tab, "<swpused-percent>%.2f</swpused-percent>",
-			smc->tlskb ?
-			SP_VALUE(smc->frskb, smc->tlskb, smc->tlskb) :
-			0.0);
-
-		xprintf(tab, "<swpcad>%llu</swpcad>",
-			smc->caskb);
-
-		xprintf(tab--, "<swpcad-percent>%.2f</swpcad-percent>",
-			(smc->tlskb - smc->frskb) ?
-			SP_VALUE(0, smc->caskb, smc->tlskb - smc->frskb) :
-			0.0);
+		xml_print_swap_memory_stats(smc, &tab);
 	}
 
 	xprintf(tab, "</memory>");
@@ -1399,7 +1398,7 @@ __print_funct_t xml_print_net_etcp_stats(struct activity *a, int curr, int tab,
 	xprintf(tab, "<net-etcp "
 		"atmptf=\"%.2f\" "
 		"estres=\"%.2f\" "
-		"retrans=\"%.2f\" "
+		"retrseg=\"%.2f\" "
 		"isegerr=\"%.2f\" "
 		"orsts=\"%.2f\"/>",
 		S_VALUE(snetp->AttemptFails, snetc->AttemptFails,  itv),
@@ -1798,7 +1797,7 @@ __print_funct_t xml_print_pwr_cpufreq_stats(struct activity *a, int curr, int ta
 		spc = (struct stats_pwr_cpufreq *) ((char *) a->buf[curr] + i * a->msize);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -2034,7 +2033,7 @@ __print_funct_t xml_print_pwr_wghfreq_stats(struct activity *a, int curr, int ta
 		spp = (struct stats_pwr_wghfreq *) ((char *) a->buf[!curr] + i * a->msize * a->nr2);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -2295,8 +2294,8 @@ __print_funct_t xml_print_softnet_stats(struct activity *a, int curr, int tab,
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* No */
 			continue;
 
